#include<stdio.h>

/***************************************************************************
 * Program      : PrintThru
 * Author       : Nicholas Clifton Barnard <nickb@pobox.com>
 * Date         : March 4, 2003
 * Description  : Prints given file through ANSI Print Through telnet software
 * Arguments    : File name to print, if none given assumes STDIN
 * Exit Value   : 0 - Successfully Printed
 *                1 - Could not open /dev/tty
 *                2 - Could not open file on command line
 * Copyright    : Copyright (C) 2003 Nicholas Barnard  
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ****************************************************************************/

char snos(int number);

main(int argc, char *argv[])
{
	char temp;
	int lines = 0;
	int len = 0;
	int chrs = 0;
	FILE *in;
	FILE *out;

	if ((out = fopen("/dev/tty", "w")) == NULL) {
		perror("/dev/tty");
		return (1);
	}
	fprintf(out, "\033[5i");
	if (argc > 1) {
		if ((in = fopen(argv[1], "r")) == NULL) {
			fprintf(out, "\033[4i");
			fclose(out);

			printf("Could not open %s\n", argv[1]);
			return (2);
		}
	} else {
		in = stdin;
	}
	while ((feof(in) == 0)) {
		temp = getc(in);
		switch (temp) {
		case '\n':
			len = 0;
			putc(temp, out);
			lines++;
			chrs = 0;
			break;
		case '\b':
			len--;
			chrs--;
			putc(temp, out);
			break;
		case '\r':
			len = 0;
			chrs = 0;
			putc(temp, out);
			break;
		case '\f':
			len = 0;
			chrs = 0;
			putc(temp, out);
			break;
		case '\t':
			len += (8 - chrs);
			putc(temp, out);
			break;
		default:
			if (len == 80) {
				len = 0;
				putc('\n', out);
				lines++;
				putc(temp, out);
			} else {
				len++;
				putc(temp, out);
			}
		}
		if (chrs == 8) {
			chrs = 0;
		}
	}

	fprintf(out, "\n%c", 12);
	fprintf(out, "\033[4i");
	fclose(out);


	if (argc > 1) {
		printf("%d line%c of %s succssfully printed to the printer attached to the terminal.\n", lines, snos(lines), argv[1]);
		printf("%d page%c printed.\n", ((lines + 60) / 60), snos((lines + 60) / 60));
		fclose(in);
	} else {
		printf("%d line%c succssfully printed from the standard input to the printer attached to the terminal.\n", lines, snos(lines));
		printf("%d page%c printed.\n", ((lines + 60) / 60), snos((lines + 60) / 60));
	}
	return (0);
}

char 
snos(int number)
{
	if (number == 1) {
		return ('\0');
	}
	return ('s');
}
